<?php
// URL a laquelle on doit soumettre l'url du fichier "sitemap.xml.gz"
define("GOOGLE_SUBMIT_URL", "http://www.google.com/webmasters/sitemaps/ping?sitemap=");
// URL de la doc Google pour creer des sitemaps multiples
define("GOOGLE_MULTIPLE_SITEMAPS", "https://www.google.com/webmasters/sitemaps/docs/en/protocol.html#sitemapFileRequirements");

// dfinition des styles CSS du premier ecran
define("CSS_SITEMAP_PAGE", "margin:20px 15%; text-align:center");
define("CSS_SITEMAP_TITRE", "margin:10px 0; font:16px verdana,arial,sans-serif; text-align:center");
define("CSS_SITEMAP_MESSAGE", "margin:10px 0; font:12px verdana,arial,sans-serif; text-align:left; color:#777;");
define("CSS_SITEMAP_FORM", "display:inline; padding:5px;");

// la fonction ob_get_clean() n'est supportee qu'a partir de PHP 4.3.0
if (!function_exists("ob_get_clean")) {
   function ob_get_clean() {
       $ob_contents = ob_get_contents();
       ob_end_clean();
       return $ob_contents;
   }
}

class Sitemap {
    var $url_site;                            // url du site
    var $dir_files = null;                    // rpertoire des fichiers sitemap (hors $gz_file :  la racine)
    var $gz_file = "sitemap.xml.gz";          // nom du fichier sitemap compresse
    var $dluo = 600;                          // date du fichier sitemap le plus rcent (en secondes)
    var $xml_size = 0;                        // taille du fichier XML                
    var $nb_urls = 0;                         // nombre d'urls
    var $erreur = null;
    
    function Sitemap($dir_files=null, $gz_file=null) {
        $chemin = $_SERVER["PHP_SELF"];
        $chemin = substr($chemin, 0, strrpos($chemin, "/"));
        $this->url_site = "http://" . $_SERVER["SERVER_NAME"] . "/$chemin";
        if (!is_null($gz_file)) $this->gz_file = $gz_file;
        if (!is_null($dir_files)) $this->dir_files = preg_replace("/\/?(w+)\/?/", "$1", $dir_files);
    }
    
    // methodes publiques
    
    function compresserXML($contenu) {
        // compresse le XML produit par le squelette dans sitemap.xml.gz
        
        // contrle si le contenu XML ne depasse pas 10 Mo et 50 000 URLs
        $this->controlerBuffer($contenu); 
        // si le fichier sitemap.xml.gz existe, controle s'il est possible de reecrire dessus 
        if (!$this->erreur) $this->controlerFichier();
        // compresse le contenu dans sitemap.xml.gz
        if (!$this->erreur) $this->compresser($contenu);

        // s'il y a eu erreur : affichage de l'erreur
        if ($this->erreur) $this->afficherErreur();
        // sinon : affichage de 2 boutons "voir XML" et "Envoi a Google"
        else $this->afficherForm($contenu);
    }

    function envoyerSitemap() {
        // rechargement de la page : envoi de sitemap.xml.gz a Google 
        if (isset($_POST["envoi"]) && ($_POST["envoi"] == "oui")) {
            // determine si le sitemap existe et si ce n'est pas une ancienne version
            $this->verifierSitemapGz();
            // si erreur : affichage de l'erreur
            if ($this->erreur) $this->afficherErreur(); 
            // sinon : envoi de l'url du fichier sitemap.xml.gz a Google 
            else {
                header("Location: " . GOOGLE_SUBMIT_URL . "$this->url_site/$this->gz_file");
            }
            return true;
        }
        else return false;
    }
    
    function setDluo($dluo) {
        // modifie la date du fichier "sitemap.xml.gz", qu'on peut considerer trop ancienne
        $this->dluo = $dluo;
    }
    
    // methodes privees

    function verifierSitemapGz() {
        // determine si le sitemap existe et si ce n'est pas une ancienne version
        if (file_exists($this->gz_file)) {
            $gz = fopen($this->gz_file, "r");
            $fstat = fstat($gz);
            fclose($gz);
            // si le fichier est plus ancien que $this->dluo, 
            // a signifie que le nouveau fichier n'a pas ete cree
            if ((time() - $fstat["mtime"]) > $this->dluo) {
                $this->erreur = 'Le nouveau fichier "$this->gz_file" n\'a pas &eacute;t&eacute; cr&eacute;&eacute;.';
            }
        }
        // il n'y a pas de fichier "sitemap.xml.gz"
        else $this->erreur = 'Le fichier "$this->gz_file" est introuvable';
    }
         
    function compresser($buffer) {
        // compresse le contenu du $buffer dans le fichier sitemap
           $gz = gzopen($this->gz_file, "w9");
           if ($gz) {
               gzwrite($gz, $buffer);
           }
           // on n'a pas pu creer le fichier "sitemap.xml.gz" : affichage de l'erreur
           else {
               $this->erreur = 'Erreur : impossible de cr&eacute;er ou d\'&eacute;crire dans le fichier $this->gz_file';
           }
           gzclose($gz);
           // si le fichier "sitemap.xml.gz" a ete cree, on le met en ecriture (pour la prochaine mise a jour)
           if (!$this->erreur) chmod($this->gz_file, 0666);
    }
    
    function controlerBuffer($buffer) {
        // contrle si le fichier sitemap.xml fait moins de 10 Mo et de 50 000 urls
        if (!$buffer) $this->erreur = 'Le buffer est vide';
        // erreur si $buffer > 10 Mo
        else {
            $this->xml_size = strlen($buffer);
            $urls = preg_split("/<url>/", $buffer);
            $this->nb_urls = count($urls) - 1;
            if ($this->xml_size > 10*1024*1024) {
                $this->erreur = 'Votre fichier sitemap fait ' . ($this->xml_size/1024/1024). ' Mo. La limite est de 10 Mo.';
            }
            // on vrifie s'il n'y a pas plus de 50 000 urls
            elseif ($this->nb_urls > 50000) {
                $this->erreur = 'Votre fichier sitemap a $this->nb_urls URLs. La limite est de 50 000.';
            }
            if ($this->erreur) {
                $this->erreur .= '<br />Vous devez le d&eacute;couper en plusieurs sitemaps et faire un sitemap-index.';
                $this->erreur .= '<br />Voir la <a href="' . GOOGLE_MULTIPLE_SITEMAPS. '">doc Google</a> &agrave; ce sujet.';
            }
        }
    }

    function controlerFichier() {
        // controle si le fichier existe et est en ecriture ou si le repertoire est autorise en ecriture
        if (file_exists ($this->gz_file)) {
               if (!is_writable($this->gz_file)) {
                   $this->erreur = 'Erreur : impossible d\'&eacute;crire dans $this->gz_file';
               }
        }
        elseif (!is_writable (dirname ($this->gz_file))) {
               $this->erreur = 'Erreur : impossible d\'&eacute;crire dans ' . dirname ($this->gz_file);
        }
    }

    function afficherErreur() {
        // affiche l'erreur generee par une des methodes de la classe
        echo "<html><body>";
        echo '<div style="margin-top:20px;text-align:center;font-weight:bold;color:#f00;">' . $this->erreur . '</div>';
        echo "</body></html>";
    }

    function afficherForm($chaine) {
        // affiche 2 boutons "Voir le XML" et "Envoyer le sitemap  Google"
        // dans 2 formulaires distincts.
        // les styles utilises sont definis en tete de ce fichier (fonctions define())
        echo "<html><body>";
        echo '<div style="' . CSS_SITEMAP_PAGE . '">';
        // titre
        echo '<div style="' . CSS_SITEMAP_TITRE . '">Lecture du Sitemap Google</div>'; 
        // message d'info
        echo '<div style="' . CSS_SITEMAP_MESSAGE . '">';
        echo 'Votre fichier XML contient ' . $this->nb_urls . ' URLs et fait ' . $this->xml_size . ' octets';
        echo '<br />Cliquez sur <b>Voir le XML</b> pour voir le XML produit dans une nouvelle fen&ecirc;tre';
        echo '<br />Cliquez sur <b>Envoyer le sitemap &agrave; Google</b> pour g&eacute;n&eacute;rer la requ&ecirc;te HTTP :';
        echo '<br />' . GOOGLE_SUBMIT_URL . $this->url_site . $this->gz_file;
        echo '</div>';
        // 1er formulaire : ouvre une nouvelle fenetre avec le XML produit
        echo '<form action="' . $this->dir_files . '/sitemap.xml.php" method="post" target="_BLANK" style="' . CSS_SITEMAP_FORM . '">';
        echo '<input type="hidden" name="fichier" value="' . $this->url_site . $this->gz_file . '" />';
        echo '<input class="spip_bouton" type="submit" value="Voir le XML" />';
        echo '</form>';
        // 2e formulaire : envoie le nom du fichier a Google
        echo '<form action="' . $_SERVER["PHP_SELF"] . '" method="post" style="' . CSS_SITEMAP_FORM . '">';
        echo '<input type="hidden" name="envoi" value="oui" />';
        echo '<input class="spip_bouton" type="submit" value="Envoyer le sitemap &agrave; Google" />';
        echo '</form>';
        echo '</div>';
        echo "</body></html>";
    }
}

?>